package org.sonatype.ahc.certs;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.net.Socket;
import java.security.Principal;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;

import javax.net.ssl.SSLEngine;
import javax.net.ssl.X509ExtendedKeyManager;
import javax.net.ssl.X509KeyManager;

/**
 * SSL key manager wrapper with the ability to force specific client aliases.
 * 
 * @author Benjamin Hanzelmann
 * @see X509ExtendedKeyManager
 */
public class AHCKeyManager
    extends X509ExtendedKeyManager
{

    private final X509ExtendedKeyManager delegate;

    private String alias;

    /**
     * Wrap the given key manager.
     * 
     * @param delegate the key manager to use for default behavior.
     */
    public AHCKeyManager( X509ExtendedKeyManager delegate )
    {
        super();
        this.delegate = delegate;
    }

    /**
     * If no specific alias is set ({@code alias == null}), use the default behavior of the delegated key manager,
     * otherwise return the alias set by {@link #setAlias(String)}.
     * 
     * @see X509ExtendedKeyManager#chooseClientAlias(String[], Principal[], Socket)
     */
    public String chooseClientAlias( String[] keyType, Principal[] issuers, Socket socket )
    {
        if ( alias != null )
        {
            return alias;
        }
        else
        {
            return delegate.chooseClientAlias( keyType, issuers, socket );
        }
    }

    public String chooseServerAlias( String arg0, Principal[] arg1, Socket arg2 )
    {
        return delegate.chooseServerAlias( arg0, arg1, arg2 );
    }

    public X509Certificate[] getCertificateChain( String arg0 )
    {
        return delegate.getCertificateChain( arg0 );
    }

    public String[] getClientAliases( String keytype, Principal[] issuers )
    {
        return delegate.getClientAliases( keytype, issuers );
    }

    public PrivateKey getPrivateKey( String alias )
    {
        return delegate.getPrivateKey( alias );
    }

    public String[] getServerAliases( String keyType, Principal[] issuers )
    {
        return delegate.getServerAliases( keyType, issuers );
    }

    /**
     * If no specific alias is set ({@code alias == null}), use the default behavior of the delegated key manager,
     * otherwise return the alias set by {@link #setAlias(String)}.
     * 
     * @see X509ExtendedKeyManager#chooseEngineClientAlias(String[], Principal[], Socket)
     */
    @Override
    public String chooseEngineClientAlias( String[] keyType, Principal[] issuers, SSLEngine engine )
    {
        if ( alias != null )
        {
            return alias;
        }
        else
        {
            return delegate.chooseEngineClientAlias( keyType, issuers, engine );
        }
    }

    @Override
    public String chooseEngineServerAlias( String keyType, Principal[] issuers, SSLEngine engine )
    {
        return super.chooseEngineServerAlias( keyType, issuers, engine );
    }

    /**
     * Sets the alias to choose for the SSL key presented to the server.
     * <p>
     * TODO some kind of sanity check, e.g. "is this alias actually contained in the key manager?"
     * 
     * @param alias the alias to choose for the presented SSL key. If {@code null}, the delegates
     *            {@link X509KeyManager#chooseClientAlias(String[], Principal[], Socket)} and
     *            {@link X509ExtendedKeyManager#chooseEngineClientAlias(String[], Principal[], SSLEngine)} will be used.
     * @see #AHCKeyManager(X509ExtendedKeyManager)
     * @see X509KeyManager#chooseClientAlias(String[], Principal[], Socket)
     * @see X509ExtendedKeyManager#chooseEngineClientAlias(String[], Principal[], SSLEngine)
     */
    public void setAlias( String alias )
    {
        this.alias = alias;
    }

}
