package org.sonatype.ahc.certs;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import javax.net.ssl.X509TrustManager;

import org.sonatype.ahc.certs.Entry.Type;

/**
 * A trust manager with the ability to override behavior in specific situations.
 * 
 * @author Benjamin Hanzelmann
 */
public class AHCTrustManager
    implements X509TrustManager
{

    private final X509TrustManager delegate;

    private boolean trustAll = false;

    private CertificateManager manager = null;

    private X509Certificate[] acceptedIssuers = null;

    public AHCTrustManager( CertificateManager manager, X509TrustManager delegate )
    {
        this.delegate = delegate;
        this.manager = manager;
    }

    public AHCTrustManager( X509TrustManager delegate )
    {
        this.delegate = delegate;
    }

    public void checkClientTrusted( X509Certificate[] arg0, String arg1 )
        throws CertificateException
    {
        delegate.checkClientTrusted( arg0, arg1 );
    }

    public void checkServerTrusted( X509Certificate[] arg0, String arg1 )
        throws CertificateException
    {
        if ( getTrustAll() )
        {
            return;
        }

        CertificateException delegateException = null;
        boolean accepted = true;

        try
        {
            if ( delegate != null )
            {
                delegate.checkServerTrusted( arg0, arg1 );
            }
        }
        catch ( CertificateException e )
        {
            delegateException = e;
            accepted = false;
        }

        // TODO provide custom rules?

        if ( !accepted )
        {
            if ( delegateException != null )
            {
                throw delegateException;
            }
            else
            {
                throw new CertificateException( "No matching certificate found." );
            }
        }
    }

    public X509Certificate[] getAcceptedIssuers()
    {
        if ( acceptedIssuers != null )
        {
            return acceptedIssuers;
        }
        else {
            return delegate.getAcceptedIssuers();
        }
    }

    public List<X509Certificate> getX509Certificates()
    {
        List<X509Certificate> accepted = new LinkedList<X509Certificate>();

        if ( manager != null )
        {
            Collection<Entry> entries = this.manager.getEntries();
            for ( Entry entry : entries )
            {
                if ( Type.CERTIFICATE.equals( entry.getType() ) )
                {
                    accepted.add( (X509Certificate) ( (AHCCertificate) entry ).getCertificate() );
                }
            }
        }

        return accepted;
    }

    /**
     * Returns {@code true} if this manager trusts all certificates, {@code false} otherwise.
     * 
     * @return {@code true} if this manager trusts all certificates, {@code false} otherwise.
     */
    public boolean getTrustAll()
    {
        return trustAll;
    }

    /**
     * Sets whether this manager should trust all certificates or delegates this decision to the underlying
     * {@link X509TrustManager}.
     * 
     * @param trustAll
     * @see X509TrustManager#checkClientTrusted(X509Certificate[], String)
     * @see X509TrustManager#checkServerTrusted(X509Certificate[], String)
     */
    public void setTrustAll( boolean trustAll )
    {
        this.trustAll = trustAll;
    }

    /**
     * Set the certificates to accept as issuers of a certificate.
     * 
     * @param acceptedIssuers the certificates to accept as issuers of a certificate.
     */
    public void setAcceptedIssuers( X509Certificate[] acceptedIssuers )
    {
        this.acceptedIssuers = acceptedIssuers;
    }

}