package org.sonatype.ahc.certs;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.security.KeyStore;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.LinkedList;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.ahc.certs.Entry.Type;
import org.sonatype.ahc.certs.exceptions.CertServiceException;

/**
 * A utility class to support handling certificates.
 * 
 * @author Benjamin Hanzelmann
 * @see CertificateManager
 */
public class Certificates
{

    private static final Logger log = LoggerFactory.getLogger( Certificates.class );

    /**
     * Creates a new KeyStore (JVM default type) with the given collection of key store entries.
     * 
     * @param entries the store entries to add to the new key store.
     * @return a new key store with the given entries in it.
     */
    public static KeyStore createKeyStore( Collection<Entry> entries )
    {
        return createKeyStore( entries, null );
    }

    /**
     * Creates a new KeyStore (JVM default type) with the given collection of key store entries.
     * 
     * @param entries the store entries to add to the new key store.
     * @param password the password to use for the store entries
     * @return a new key store with the given entries in it.
     */
    public static KeyStore createKeyStore( Collection<Entry> entries, char[] password )
    {
        if ( password == null )
        {
            password = Session.PASSWORD;
        }
        KeyStore store = null;
        try
        {
            store = KeyStore.getInstance( KeyStore.getDefaultType() );
            store.load( null, password );
            for ( Entry entry : entries )
            {
                log.debug( String.format( "Adding %s: '%s'", entry.getType().toString(), entry.getAlias() ) );
                switch ( entry.getType() )
                {
                    case KEY:
                        AHCKey key = (AHCKey) entry;
                        store.setKeyEntry( key.getAlias(), key.getPrivateKey(), password,
                                           unpackCertificates( key.getCertificateChain() ) );
                        AHCCertificate[] chain = key.getCertificateChain();
                        for ( int i = 1; i < chain.length; i++ )
                        {
                            AHCCertificate ahcCert = chain[i];
                            X509Certificate x509Certificate = (X509Certificate) ahcCert.getCertificate();
                            store.setCertificateEntry( x509Certificate.getSubjectDN().getName(), x509Certificate );
                        }
                        break;
                    case CERTIFICATE:
                        AHCCertificate cert = (AHCCertificate) entry;
                        store.setCertificateEntry( cert.getAlias(), cert.getCertificate() );
                        break;
                    default:
                        break;
                }
            }
        }
        catch ( Exception e )
        {
            throw new CertServiceException( e.getMessage(), e );
        }
        return store;
    }

    public static Certificate[] unpackCertificates( AHCCertificate[] chain )
    {
        Certificate[] certs = new Certificate[chain.length];
        for ( int i = 0; i < chain.length; i++ )
        {
            certs[i] = chain[i].getCertificate();
        }
        return certs;
    }
    
    public static Collection<String> getAliases( Collection<Entry> entries, Type type )
    {
        Collection<String> aliases = new LinkedList<String>();
        for ( Entry entry : entries )
        {
            if ( type != null && entry.getType().equals( type ) )
            {
                aliases.add( entry.getAlias() );
            }
        }
        return aliases;
    }

}
