package org.sonatype.ahc.certs;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.security.KeyStore;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Queue;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509ExtendedKeyManager;
import javax.net.ssl.X509TrustManager;

import org.codehaus.plexus.component.annotations.Component;
import org.codehaus.plexus.component.annotations.Requirement;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.ahc.certs.Entry.Type;
import org.sonatype.ahc.certs.exceptions.CertServiceException;

import com.ning.http.client.AsyncHttpClientConfig;
import com.ning.http.client.AsyncHttpClientConfig.Builder;
import com.ning.http.client.filter.FilterContext;
import com.ning.http.client.filter.FilterException;
import com.ning.http.client.filter.ResponseFilter;
import com.ning.http.client.resumable.ResumableIOExceptionFilter;

/**
 * @author Benjamin Hanzelmann
 */
@Component( role = Configurator.class )
public class DefaultConfigurator
    implements Configurator
{

    @Requirement
    private CertificateManager manager;

    private SSLConfig params = new SSLConfig.Builder().build();

    private AsyncHttpClientConfig ahcConfig = new AsyncHttpClientConfig.Builder().build();

    public void setCertificateManager( CertificateManager manager )
    {
        this.manager = manager;
    }

    public DefaultConfigurator()
    {
        // enable default constructor
    }

    public DefaultConfigurator( CertificateManager manager )
    {
        setCertificateManager( manager );
    }

    public AsyncHttpClientConfig build( AsyncHttpClientConfig prototype )
    {
        return build( manager, prototype );
    }

    private SSLContextValues sslContext( CertificateManager manager )
    {
        Collection<Entry> entries = manager.getEntries();
        if ( entries == null )
        {
            throw new IllegalArgumentException(
                                                "CertificateManager is unconfigured (CertificateManager#getEntries() may not return null)" );
        }

        SSLContextValues values = new SSLContextValues();
        KeyStore store = Certificates.createKeyStore( entries );
        KeyManagerFactory keyManagerFactory;
        try
        {
            keyManagerFactory = KeyManagerFactory.getInstance( "SunX509" );
            keyManagerFactory.init( store, Session.PASSWORD );
            KeyManager[] keyManagers = keyManagerFactory.getKeyManagers();
            AHCKeyManager[] ahcKeyManagers = new AHCKeyManager[keyManagers.length];
            for ( int i = 0; i < keyManagers.length; i++ )
            {
                if ( keyManagers[i] instanceof X509ExtendedKeyManager )
                {
                    AHCKeyManager ahcKeyManager = new AHCKeyManager( (X509ExtendedKeyManager) keyManagers[i] );
                    keyManagers[i] = ahcKeyManager;
                    ahcKeyManagers[i] = ahcKeyManager;
                }
            }

            values.setKeyManagers( ahcKeyManagers );

            TrustManagerFactory tmf = TrustManagerFactory.getInstance( TrustManagerFactory.getDefaultAlgorithm() );
            tmf.init( store );
            TrustManager[] trustManagers = tmf.getTrustManagers();
            AHCTrustManager[] ahcTrustManagers = new AHCTrustManager[trustManagers.length];
            for ( int i = 0; i < trustManagers.length; i++ )
            {
                TrustManager tm = trustManagers[i];
                if ( tm instanceof X509TrustManager )
                {
                    AHCTrustManager ahcTrustManager = new AHCTrustManager( manager, (X509TrustManager) tm );
                    ahcTrustManager.setTrustAll( params.isTrustAllCerts() );
                    trustManagers[i] = ahcTrustManager;
                    ahcTrustManagers[i] = ahcTrustManager;
                }
            }

            values.setTrustManagers( ahcTrustManagers );

            SSLContext context = SSLContext.getInstance( "TLS" );
            context.init( keyManagers, trustManagers, null );

            values.setSSLContext( context );

            return values;
        }
        catch ( Exception e )
        {
            throw new CertServiceException( "Could not create custom SSL context: " + e.toString(), e );
        }
    }

    public AsyncHttpClientConfig build( CertificateManager manager )
    {
        return build( manager, null );
    }

    public AsyncHttpClientConfig build( CertificateManager manager, AsyncHttpClientConfig prototype )
    {
        Builder builder;
        if ( prototype != null )
        {
            builder = new Builder( prototype );
        }
        else
        {
            builder = new Builder( ahcConfig );
        }

        if ( manager.getEntries() == null )
        {
            throw new IllegalArgumentException(
                                                "CertificateManager is unconfigured (CertificateManager#getEntries() may not return null)" );
        }

        SSLContextValues sslValues;
        if ( !manager.getEntries().isEmpty() )
        {
            sslValues = sslContext( manager );
            builder.setSSLContext( sslValues.getSSLContext() );
            SSLKeyChooser responseFilter =
                new SSLKeyChooser( Certificates.getAliases( manager.getEntries(), Type.KEY ),
                                   new ManagerAggregate( sslValues.getKeyManagers(), sslValues.getTrustManagers() ) );
            builder.addResponseFilter( responseFilter );
        }

        if ( params.isResume() )
        {
            builder.addIOExceptionFilter( new ResumableIOExceptionFilter() );
        }

        return builder.build();
    }

    public AsyncHttpClientConfig build()
    {
        return build( manager );
    }

    /**
     * Set the SSLParameters to use.
     * 
     * @param parameters the SSL parameters to use. If {@code null}, use default parameters.
     * @see SSLConfig#DEFAULT
     */
    public void setSSLConfig( SSLConfig parameters )
    {
        if ( parameters == null )
        {
            parameters = new SSLConfig.Builder().build();
        }

        this.params = parameters;
    }

    /**
     * Return the currently used SSLParameters.
     * 
     * @return
     */
    public SSLConfig getSSLParameters()
    {
        return this.params;
    }

    public AsyncHttpClientConfig getAHCConfig()
    {
        return ahcConfig;
    }

    public void setAHCConfig( AsyncHttpClientConfig config )
    {
        this.ahcConfig = config;
    }

    class SSLContextValues
    {

        private SSLContext sslContext;

        private AHCKeyManager[] keyManagers;

        private AHCTrustManager[] trustManagers;

        public SSLContext getSSLContext()
        {
            return sslContext;
        }

        public void setSSLContext( SSLContext sslContext )
        {
            this.sslContext = sslContext;
        }

        public void setKeyManagers( AHCKeyManager[] keyManagers )
        {
            this.keyManagers = keyManagers;
        }

        public void setTrustManagers( AHCTrustManager[] trustManagers )
        {
            this.trustManagers = trustManagers;
        }

        public AHCKeyManager[] getKeyManagers()
        {
            return keyManagers;
        }

        public AHCTrustManager[] getTrustManagers()
        {
            return trustManagers;
        }

    }

    /**
     * An AHC ResponseFilter that iterates through SSL keys on HTTP status 403.
     * 
     * @author Benjamin Hanzelmann
     */
    class SSLKeyChooser
        implements ResponseFilter
    {

        private final Logger logger = LoggerFactory.getLogger( getClass() );

        private final Queue<String> aliases;

        private String alias;

        private final ManagerAggregate keyManagerAggregate;

        /**
         * Create a key chooser to use with an AHC connection.
         * 
         * @param aliases the aliases to try.
         * @param keyManagerAggregate the aggregation of used key managers.
         * @see AHCKeyManager#setAlias(String)
         */
        public SSLKeyChooser( Collection<String> aliases, ManagerAggregate keyManagerAggregate )
        {
            logger.debug( "Creating SSLKeyChooser with aliases: " + aliases.toString() );
            this.aliases = new LinkedList<String>( aliases );
            this.alias = this.aliases.poll();
            this.keyManagerAggregate = keyManagerAggregate;
        }

        /**
         * If the response status is 403, retry the connection with the next configured alias. Otherwise just let it
         * pass.
         */
        @SuppressWarnings( { "rawtypes" } )
        public FilterContext filter( FilterContext ctx )
            throws FilterException
        {
            int code = ctx.getResponseStatus().getStatusCode();
            String msg = ctx.getResponseStatus().getStatusText();
            logger.debug( String.format( "Response received (%s, %s) ", code, msg ) );
            if ( code == 403 )
            {
                logger.debug( "Forbidden: " + msg );
                if ( alias != null )
                {
                    logger.debug( "Trying next alias: " + alias );
                    keyManagerAggregate.setAlias( alias );
                    alias = aliases.poll();
                    return new FilterContext.FilterContextBuilder( ctx ).replayRequest( true ).build();
                }
                logger.debug( "No alias left, won't replay connection." );
            }
            return ctx;
        }

    }

}
