package org.sonatype.ahc.certs.extractor;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.codehaus.plexus.component.annotations.Component;
import org.sonatype.ahc.certs.Entry;
import org.sonatype.ahc.certs.exceptions.ExtractionException;
import org.sonatype.ahc.certs.extractor.certfile.PKCS12CertificateExtractor;
import org.sonatype.ahc.certs.extractor.keystore.KeystoreCertificateExtractor;

/**
 * @author Benjamin Hanzelmann
 */
@Component( role = CertificateExtractorUtility.class )
public class DefaultCertificateExtractorUtility
    implements CertificateExtractorUtility
{

    private static final class PriorityComparator
        implements Comparator<CertificateExtractor>
    {
        public static final Comparator<CertificateExtractor> INSTANCE = new PriorityComparator();

        public int compare( CertificateExtractor o1, CertificateExtractor o2 )
        {
            return new Integer( o1.getPriority() ).compareTo( new Integer( o2.getPriority() ) );
        }
    }

    private final List<CertificateExtractor> extractors = new LinkedList<CertificateExtractor>();

    public DefaultCertificateExtractorUtility()
    {
        super();
        init();
    }

    private void init()
    {
        this.extractors.add( new KeystoreCertificateExtractor() );
        this.extractors.add( new PKCS12CertificateExtractor() );
    }

    public void register( CertificateExtractor extractor )
    {
        extractors.add( extractor );
        Collections.sort( extractors, PriorityComparator.INSTANCE );
    }

    public void register( Collection<CertificateExtractor> extractor )
    {
        extractors.addAll( extractor );
        Collections.sort( extractors, PriorityComparator.INSTANCE );
    }

    public void remove( CertificateExtractor extractor )
    {
        extractors.remove( extractor );
    }

    public Collection<CertificateExtractor> getExtractors()
    {
        return Collections.unmodifiableCollection( extractors );
    }

    public Collection<CertificateExtractor> getExtractors( Class<?> cls )
    {
        return Collections.unmodifiableCollection( filter( cls, extractors ) );
    }

    private Collection<CertificateExtractor> filter( Class<?> cls, List<CertificateExtractor> list )
    {
        List<CertificateExtractor> filtered = new LinkedList<CertificateExtractor>();

        for ( CertificateExtractor item : list )
        {
            if ( item.canHandle( cls ) )
            {
                filtered.add( item );
            }
        }

        return filtered;
    }

    public Collection<Entry> extract( Object src, Map<String, Object> hints )
        throws ExtractionException
    {
        Collection<Entry> entries = null;
        List<ExtractionException> exceptions = new LinkedList<ExtractionException>();

        Collection<CertificateExtractor> filtered = getExtractors( src.getClass() );
        if ( filtered.isEmpty() )
        {
            throw new ExtractionException( "Could not find any handling extractors for " + src.getClass());
        }
        
        for ( CertificateExtractor extractor : filtered )
        {
            try
            {
                entries = extractor.extract( src, hints );
            }
            catch ( ExtractionException e )
            {
                exceptions.add( e );
            }
        }
        
        if ( entries == null )
        {
            throw new ExtractionException( String.format( "No successful extraction (Used %s).", filtered ),
                                           exceptions.toArray( new ExtractionException[exceptions.size()] ) );
        }

        return Collections.unmodifiableCollection(entries);
    }

    public boolean canHandle( Class<?> cls )
    {
        return !getExtractors( cls ).isEmpty();
    }

    public int getPriority()
    {
        return 0;
    }

}
