package org.sonatype.ahc.certs.extractor.certfile;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.security.GeneralSecurityException;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.ssl.PKCS8Key;
import org.codehaus.plexus.component.annotations.Component;
import org.sonatype.ahc.certs.AHCKey;
import org.sonatype.ahc.certs.DefaultCertificate;
import org.sonatype.ahc.certs.DefaultKey;
import org.sonatype.ahc.certs.Entry;
import org.sonatype.ahc.certs.Session;
import org.sonatype.ahc.certs.exceptions.ExtractionException;
import org.sonatype.ahc.certs.extractor.CertificateExtractor;

/**
 * @author Benjamin Hanzelmann
 */
@Component( role = CertificateExtractor.class, hint = "pkcs8" )
public class PKCS8CertificateExtractor
    extends Session
    implements CertificateExtractor
{

    public Collection<Entry> extract( Object src, Map<String, Object> hints )
        throws ExtractionException
    {
        if ( !File.class.isAssignableFrom( src.getClass() ) )
        {
            throw new ExtractionException( "Cannot handle " + src.getClass() );
        }
        
        File certFile = File.class.cast(src);
        String name = certFile.getName().toLowerCase( Locale.ENGLISH );
        if ( !( name.endsWith( ".pem" ) || name.endsWith( ".der" ) ) )
        {
            throw new ExtractionException( "Can only handle PEM/DER files" );
        }

        Collection<Entry> entries = new LinkedList<Entry>();

        RandomAccessFile in = null;
        try
        {
            byte[] b = new byte[(int) certFile.length()];
            in = new RandomAccessFile( certFile, "r" );
            in.readFully( b );
            char[] password = hints.get( "password" ).toString().toCharArray();
            PKCS8Key key = new PKCS8Key( b, password );
            AHCKey ahcKey =
                new DefaultKey( alias( certFile, hints ), key.getPrivateKey(), PASSWORD, new DefaultCertificate[0] );

            entries.add( ahcKey );
        }
        catch ( IOException e )
        {
            throw new ExtractionException( "Problem reading file: " + e.getMessage(), e );
        }
        catch ( GeneralSecurityException e )
        {
            throw new ExtractionException( "Problem extracting key: " + e.getMessage(), e );
        }
        finally
        {
            if ( in != null )
            {
                try
                {
                    in.close();
                }
                catch ( IOException e )
                {
                    // we tried
                }
            }
        }

        return Collections.unmodifiableCollection(entries);
    }

    private String alias( File certFile, Map<String, Object> hints )
        throws IOException
    {
        if ( hints.containsKey( "alias" ) )
        {
            return hints.get( "alias" ).toString();
        }
        else
        {
            return certFile.getCanonicalPath();
        }
    }

    public boolean canHandle( Class<?> cls )
    {
        return File.class.isAssignableFrom( cls );
    }

    public int getPriority()
    {
        return 0;
    }

}
