package org.sonatype.ahc.certs.extractor.keystore;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.security.KeyStore;
import java.security.KeyStore.PrivateKeyEntry;
import java.security.KeyStore.TrustedCertificateEntry;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.codehaus.plexus.component.annotations.Component;
import org.sonatype.ahc.certs.AHCCertificate;
import org.sonatype.ahc.certs.DefaultCertificate;
import org.sonatype.ahc.certs.DefaultKey;
import org.sonatype.ahc.certs.Entry;
import org.sonatype.ahc.certs.Session;
import org.sonatype.ahc.certs.exceptions.ExtractionException;
import org.sonatype.ahc.certs.extractor.CertificateExtractor;

/**
 * @author Benjamin Hanzelmann
 */
@Component( role = CertificateExtractor.class, hint = "jks" )
public class KeystoreCertificateExtractor
    extends Session
    implements CertificateExtractor
{

    public Collection<Entry> extract( Object src, Map<String, Object> hints )
        throws ExtractionException
    {
        if ( !KeyStore.class.isAssignableFrom( src.getClass() ) )
        {
            throw new ExtractionException( "Cannot handle " + src.getClass().toString() );
        }
        
        List<Entry> entries = new LinkedList<Entry>();

        try
        {
            KeyStore store = (KeyStore) src;
            Enumeration<String> aliases = store.aliases();
            while ( aliases.hasMoreElements())
            {
                
                String alias = aliases.nextElement();
                if ( store.entryInstanceOf( alias, PrivateKeyEntry.class ))
                {
	                char[] pwChars = null;
	
	                Object password = hints.get( alias );
	                if ( password != null )
	                {
	                    pwChars = password.toString().toCharArray();
	                }
                    else
                    {
                        Object pw = hints.get( "password" );
                        pwChars = ( pw != null ? pw.toString() : "" ).toCharArray();
                    }
	
                    PrivateKey key = (PrivateKey) store.getKey( alias, pwChars );
                    // TODO: do we need AHCCertificate-chain? Rather leave it unchanged?
                    AHCCertificate[] chain = transform( alias, store.getCertificateChain( alias ) );

                    entries.add( new DefaultKey( alias, key, PASSWORD, chain ) );
                }
                else if ( store.entryInstanceOf( alias, TrustedCertificateEntry.class )){
                    Certificate certificate = store.getCertificate( alias );
                    entries.add( new DefaultCertificate( alias, certificate ) );
                }
                
            }
        }
        catch ( Exception e )
        {
            throw new ExtractionException( "Could not extract entries: " + e.getMessage(), e );
        }

        return Collections.unmodifiableCollection(entries);
    }

    /**
     * TODO: do we need AHCCertificate-chain? Rather leave it unchanged?
     * 
     * @param alias
     * @param chain
     * @return
     */
    private AHCCertificate[] transform( String alias, Certificate[] chain )
    {
        AHCCertificate[] ret = new AHCCertificate[chain.length];
        for ( int i = 0; i < chain.length; i++ )
        {
            ret[i] = new DefaultCertificate( alias, chain[i] );
        }
        return ret;
    }

    public boolean canHandle( Class<?> cls )
    {
        return KeyStore.class.isAssignableFrom( cls );
    }

    public int getPriority()
    {
        return 0;
    }

}
